@@ -2,7 +2,7 @@
 
 set -o pipefail
 shopt -s failglob
-set -u 
+set -u
 
 # Allow us to debug what's happening in the script if necessary
 if [ "${STEAM_DEBUG-}" ]; then
@@ -43,6 +43,14 @@ cd "$STEAMROOT"
 export SYSTEM_PATH="$PATH"
 export SYSTEM_LD_LIBRARY_PATH="${LD_LIBRARY_PATH-}"
 
+# Prefer host zenity binary if available
+export SYSTEM_ZENITY="$(which zenity 2>/dev/null)"
+if [ -z ${SYSTEM_ZENITY} ]; then
+	export STEAM_ZENITY="zenity"
+else
+	export STEAM_ZENITY="${SYSTEM_ZENITY}"
+fi
+
 function show_message()
 {
 	style=$1
@@ -64,7 +72,7 @@ function show_message()
 	echo -e "$title: $*"
 
 	if [ -z "$STEAMOS" ]; then
-		if ! zenity "$style" --text="$*" 2>/dev/null; then
+		if ! "${STEAM_ZENITY}" "$style" --text="$*" 2>/dev/null; then
 			# Save the prompt in a temporary file because it can have newlines in it
 			tmpfile="$(mktemp || echo "/tmp/steam_message.txt")"
 			echo -e "$*" >"$tmpfile"
@@ -141,10 +149,10 @@ function detect_platform()
 function detect_universe()
 {
 	if test -f "$STEAMROOT/Steam.cfg" && \
-	     egrep '^[Uu]niverse *= *[Bb]eta$' "$STEAMROOT/Steam.cfg" >/dev/null; then
+		 egrep '^[Uu]niverse *= *[Bb]eta$' "$STEAMROOT/Steam.cfg" >/dev/null; then
 		STEAMUNIVERSE="Beta"
 	elif test -f "$STEAMROOT/steam.cfg" && \
-	     egrep '^[Uu]niverse *= *[Bb]eta$' "$STEAMROOT/steam.cfg" >/dev/null; then
+		 egrep '^[Uu]niverse *= *[Bb]eta$' "$STEAMROOT/steam.cfg" >/dev/null; then
 		STEAMUNIVERSE="Beta"
 	else
 		STEAMUNIVERSE="Public"
@@ -207,7 +215,7 @@ function install_bootstrap()
 		tar xf "$STEAMBOOTSTRAPARCHIVE"
 		STATUS=$?
 	else
-		show_message --error $"Couldn't start bootstrap and couldn't reinstall from $STEAMBOOTSTRAPARCHIVE.  Please contact technical support."
+		show_message --error $"Couldn't start bootstrap and couldn't reinstall from $STEAMBOOTSTRAPARCHIVE.	 Please contact technical support."
 		STATUS=1
 	fi
 
@@ -219,211 +227,38 @@ function install_bootstrap()
 
 function pin_newer_runtime_libs ()
 {
-	# Set separator to newline just for this function
-	local IFS=$(echo -en "\n\b")
-	
 	# First argument is the runtime path
-	steam_runtime_path=`realpath $1`
-	
-	if [[ ! -d "$steam_runtime_path" ]]; then return; fi
-	
-	# Associative array; indices are the SONAME, values are final path
-	declare -A host_libraries_32
-	declare -A host_libraries_64
-	
-	rm -rf "$steam_runtime_path/pinned_libs_32"
-	rm -rf "$steam_runtime_path/pinned_libs_64"
-	
-	# First, grab the list of system libraries from ldconfig and put them in the arrays
-	for ldconfig_output in `/sbin/ldconfig -XNv 2> /dev/null`
-	do
-		# If line starts with a leading / and contains :, it's a new path prefix
-		if [[ "$ldconfig_output" =~ ^/.*: ]]
-		then
-			library_path_prefix=`echo $ldconfig_output | cut -d: -f1`
-		else
-			# Otherwise it's a soname symlink -> library pair, build a full path to the soname link
-			leftside=${ldconfig_output% -> *}
-			soname=`echo $leftside | tr -d '[:space:]'`
-			soname_fullpath=$library_path_prefix/$soname
-			
-			# Left side better be a symlink
-			if [[ ! -L $soname_fullpath ]]; then continue; fi
-			
-			# Left-hand side of soname symlink should be *.so.%d
-			if [[ ! $soname_fullpath =~ .*\.so.[[:digit:]]+$ ]]; then continue; fi
-			
-			final_library=`readlink -f $soname_fullpath`
-			
-			# Target library must be named *.so.%d.%d.%d
-			if [[ ! $final_library =~ .*\.so.[[:digit:]]+.[[:digit:]]+.[[:digit:]]+$ ]]; then continue; fi
-			
-			# If it doesn't exist, skip as well
-			if [[ ! -f $final_library ]]; then continue; fi
-			
-			# Save into bitness-specific associative array with only SONAME as left-hand
-			if [[ `file -L $final_library` == *"32-bit"* ]]
-			then
-				host_libraries_32[$soname]=$soname_fullpath
-			elif [[ `file -L $final_library` == *"64-bit"* ]]
-			then
-				host_libraries_64[$soname]=$soname_fullpath
-			fi
-		fi
-	done
-	
-	mkdir "$steam_runtime_path/pinned_libs_32"
-	mkdir "$steam_runtime_path/pinned_libs_64"
-	
-	for find_output in `find "$steam_runtime_path" -type l | grep \\\.so`
-	do
-		# Left-hand side of soname symlink should be *.so.%d
-		if [[ ! $find_output =~ .*\.so.[[:digit:]]+$ ]]; then continue; fi
-		
-		soname_symlink=$find_output
-		
-		final_library=`readlink -f $soname_symlink`
-		
-		# Target library must be named *.so.%d.%d.%d
-		if [[ ! $final_library =~ .*\.so.([[:digit:]]+).([[:digit:]]+).([[:digit:]]+)$ ]]; then continue; fi
-		
-		# This pattern strips leading zeroes, which could otherwise cause bash to interpret the value as binary/octal below
-		r_lib_major=$((10#${BASH_REMATCH[1]}))
-		r_lib_minor=$((10#${BASH_REMATCH[2]}))
-		r_lib_third=$((10#${BASH_REMATCH[3]}))
-		
-		# If it doesn't exist, skip as well
-		if [[ ! -f $final_library ]]; then continue; fi
-		
-		host_library=""
-		host_soname_symlink=""
-		bitness="unknown"
-		
-		soname=$(basename "$soname_symlink")
-		
-		# If we had entries in our arrays, get them
-		if [[ `file -L $final_library` == *"32-bit"* ]]
-		then
-			if [ ! -z ${host_libraries_32[$soname]+isset} ]
-			then
-				host_soname_symlink=${host_libraries_32[$soname]}
-			fi
-			bitness="32"
-		elif [[ `file -L $final_library` == *"64-bit"* ]]
-		then
-			if [ ! -z ${host_libraries_64[$soname]+isset} ]
-			then
-				host_soname_symlink=${host_libraries_64[$soname]}
-			fi
-			bitness="64"
-		fi
-		
-		# Do we have a host library found for the same SONAME?
-		if [[ ! -f $host_soname_symlink || $bitness == "unknown" ]]; then continue; fi
-		
-		host_library=`readlink -f $host_soname_symlink`
-		
-		if [[ ! -f $host_library ]]; then continue; fi
-		
-		#echo $soname ${host_libraries[$soname]} $r_lib_major $r_lib_minor $r_lib_third
+	steam_runtime_path=$(realpath "$1")
 
-		# Pretty sure the host library already matches, but we need the rematch anyway
-		if [[ ! $host_library =~ .*\.so.([[:digit:]]+).([[:digit:]]+).([[:digit:]]+)$ ]]; then continue; fi
-		
-		h_lib_major=$((10#${BASH_REMATCH[1]}))
-		h_lib_minor=$((10#${BASH_REMATCH[2]}))
-		h_lib_third=$((10#${BASH_REMATCH[3]}))
-		
-		runtime_version_newer="no"
-		
-		if [[ $h_lib_major -lt $r_lib_major ]]; then
-			runtime_version_newer="yes"
-		fi
-		
-		if [[ $h_lib_major -eq $r_lib_major && $h_lib_minor -lt $r_lib_minor ]]; then
-			runtime_version_newer="yes"
-		fi
-		
-		if [[ $h_lib_major -eq $r_lib_major && $h_lib_minor -eq $r_lib_minor && $h_lib_third -lt $r_lib_third ]]; then
-			runtime_version_newer="yes"
-		fi
-		
-		# There's a set of libraries that have to work together to yield a working dock
-		# We're reasonably convinced our set works well, and only pinning a handful would
-		# induce a mismatch and break the dock, so always pin all of these for Steam (32-bit)
-		if [[ $bitness == "32" ]]
-		then
-			if [[ 	"$soname" == "libgtk-x11-2.0.so.0"  || \
-					"$soname" == "libdbusmenu-gtk.so.4"  || \
-					"$soname" == "libdbusmenu-glib.so.4" || \
-					"$soname" == "libdbus-1.so.3" ]]
-			then
-				runtime_version_newer="yes"
-			fi
-		fi
-		
-		
-		if [[ $runtime_version_newer == "yes" ]]; then
-			echo Found newer runtime version for $bitness-bit $soname. Host: $h_lib_major.$h_lib_minor.$h_lib_third Runtime: $r_lib_major.$r_lib_minor.$r_lib_third 
-			ln -s "$final_library" "$steam_runtime_path/pinned_libs_$bitness/$soname"
-			# Keep track of the exact version name we saw on the system at pinning time to check later
-			echo "$host_soname_symlink" > "$steam_runtime_path/pinned_libs_$bitness/system_$soname"
-			echo "$host_library" >> "$steam_runtime_path/pinned_libs_$bitness/system_$soname"
-			touch "$steam_runtime_path/pinned_libs_$bitness/has_pins"
-		fi
-	done
+	if [[ ! -d "$steam_runtime_path" ]]; then
+		return
+	fi
+
+	if [ ! -x "$steam_runtime_path/setup.sh" ]; then
+		return
+	fi
+
+	# The pinning logic is in the runtime
+	"$steam_runtime_path/setup.sh" --force
+	return $?
 }
 
 function check_pins ()
 {
-	# Set separator to newline just for this function
-	local IFS=$(echo -en "\n\b")
-	
 	# First argument is the runtime path
-	steam_runtime_path=`realpath $1`
-	
-	if [[ ! -d "$steam_runtime_path" ]]; then return; fi
-	
-	pins_need_redoing="no"
-	
-	# If we had the runtime previously unpacked but never ran the pin code, do it now
-	if [[ ! -d "$steam_runtime_path/pinned_libs_32" ]]
-	then
-		pins_need_redoing="yes"
+	steam_runtime_path=$(realpath "$1")
+
+	if [[ ! -d "$steam_runtime_path" ]]; then
+		return
 	fi
-	
-	if [[ -f "$steam_runtime_path/pinned_libs_32/has_pins" || -f "$steam_runtime_path/pinned_libs_64/has_pins" ]]
-	then
-		for pin in "$steam_runtime_path"/pinned_libs_*/system_*
-		do
-			host_sonamesymlink=`head -1 "$pin"`
-			host_library=`tail -1 "$pin"`
-			
-			# Follow the host SONAME symlink we saved in the first line of the pin entry
-			host_actual_library=`readlink -f $host_sonamesymlink`
-			
-			# It might not exist anymore if it got uninstalled or upgraded to a different major version
-			if [[ ! -f $host_actual_library ]]
-			then
-				pins_need_redoing="yes"
-			fi
-			
-			# We should end up at the same lib we saved in the second line
-			if [[ $host_actual_library != $host_library  ]]
-			then
-				# Mismatch, it could have gotten upgraded
-				pins_need_redoing="yes"
-			fi
-		done
-	fi
-	
-	if [[ $pins_need_redoing == "yes" ]]
-	then
-		echo Pins potentially out-of-date, rebuilding...
-		pin_newer_runtime_libs "$steam_runtime_path"
-	else
-		echo Pins up-to-date!
+
+	if [ ! -x "$steam_runtime_path/setup.sh" ]; then
+		return
 	fi
+
+	# The pinning logic is in the runtime
+	"$steam_runtime_path/setup.sh"
+	return $?
 }
 
 function runtime_supported()
@@ -444,7 +279,7 @@ function runtime_supported()
 
 function download_archive()
 {
-	curl -#Of "$2" 2>&1 | tr '\r' '\n' | sed 's,[^0-9]*\([0-9]*\).*,\1,' | zenity --progress --auto-close --no-cancel --width 400 --text="$1\n$2"
+	curl -#Of "$2" 2>&1 | tr '\r' '\n' | sed 's,[^0-9]*\([0-9]*\).*,\1,' | "${STEAM_ZENITY}" --progress --auto-close --no-cancel --width 400 --text="$1\n$2"
 	return ${PIPESTATUS[0]}
 }
 
@@ -462,7 +297,7 @@ function extract_archive()
 		;;
 	esac
 	if [ "${BF}" ]; then
-		tar --blocking-factor=${BF} --checkpoint=1 --checkpoint-action='exec=echo $TAR_CHECKPOINT' -xf "$2" -C "$3" | zenity --progress --auto-close --no-cancel --width 400 --text="$1"
+		tar --blocking-factor=${BF} --checkpoint=1 --checkpoint-action='exec=echo $TAR_CHECKPOINT' -xf "$2" -C "$3" | "${STEAM_ZENITY}" --progress --auto-close --no-cancel --width 400 --text="$1"
 		return ${PIPESTATUS[0]}
 	else
 		echo "$1"
@@ -474,7 +309,7 @@ function extract_archive()
 function has_runtime_archive()
 {
 	# Make sure we have files to unpack
-    if [ ! -f "$STEAM_RUNTIME.$ARCHIVE_EXT.part0" ]; then
+	if [ ! -f "$STEAM_RUNTIME.$ARCHIVE_EXT.part0" ]; then
 		return 1
 	fi
 
@@ -644,16 +479,16 @@ function reset_steam()
 
 function steamos_arg()
 {
-    for option in "$@"
-    do
+	for option in "$@"
+	do
 		if [ "$option" = "-steamos" ]; then
 			return 0; # 0 == true in bash
-        fi
-    done
+		fi
+	done
 
 	return 1; # 1 == false in bash speak
 }
- 
+
 #determine platform
 UNAME=`uname`
 if [ "$UNAME" != "Linux" ]; then
@@ -811,16 +646,17 @@ fi
 if [ "$STEAM_RUNTIME" -a "$STEAM_RUNTIME" != "0" ]; then
 	# Unpack the runtime if necessary
 	if unpack_runtime; then
-		case $(uname -m) in
-			*64)
-				export PATH="$STEAM_RUNTIME/amd64/bin:$STEAM_RUNTIME/amd64/usr/bin:$PATH"
-				;;
-			*)
-				export PATH="$STEAM_RUNTIME/i386/bin:$STEAM_RUNTIME/i386/usr/bin:$PATH"
-				;;
-		esac
-		
-		export STEAM_RUNTIME_LIBRARY_PATH=`$STEAM_RUNTIME/run.sh --print-steam-runtime-library-paths`
+		if [ ! -x "$STEAM_RUNTIME/setup.sh" ]; then
+			echo "internal error: $STEAM_RUNTIME/setup.sh is missing, this runtime is invalid or corrupted"
+			exit 1
+		fi
+		export PATH="$("$STEAM_RUNTIME/setup.sh" --print-bin-path):$PATH"
+
+		if [ ! -x "$STEAM_RUNTIME/run.sh" ]; then
+			echo "internal error: $STEAM_RUNTIME/run.sh is missing, this runtime is invalid or corrupted"
+			exit 1
+		fi
+		export STEAM_RUNTIME_LIBRARY_PATH="$("$STEAM_RUNTIME/run.sh" --print-steam-runtime-library-paths)"
 
 		export LD_LIBRARY_PATH="$STEAM_RUNTIME_LIBRARY_PATH:${LD_LIBRARY_PATH-}"
 	else
@@ -829,6 +665,14 @@ if [ "$STEAM_RUNTIME" -a "$STEAM_RUNTIME" != "0" ]; then
 	fi
 fi
 
+if [ "${1-}" = "--run" ]; then
+	STEAM_RUNTIME_RUN_SCRIPT="$STEAM_RUNTIME/run.sh"
+	shift
+	exec "$STEAM_RUNTIME/run.sh" "$@"
+	echo "Couldn't find run.sh" >&1
+	exit 255
+fi
+
 # prepend our lib path to LD_LIBRARY_PATH
 export LD_LIBRARY_PATH="$STEAMROOT/$PLATFORM:$STEAMROOT/$PLATFORM/panorama:${LD_LIBRARY_PATH-}"
 
@@ -854,7 +698,7 @@ unset DEBUGGER # Don't use debugger if Steam launches itself recursively
 if [ "$STEAM_DEBUGGER" == "gdb" ] || [ "$STEAM_DEBUGGER" == "cgdb" ]; then
 	ARGSFILE=$(mktemp $USER.steam.gdb.XXXX)
 
-	# Set the LD_PRELOAD varname in the debugger, and unset the global version. 
+	# Set the LD_PRELOAD varname in the debugger, and unset the global version.
 	: "${LD_PRELOAD=}"
 	if [ "$LD_PRELOAD" ]; then
 		echo set env LD_PRELOAD=$LD_PRELOAD >> "$ARGSFILE"
@@ -862,16 +706,18 @@ if [ "$STEAM_DEBUGGER" == "gdb" ] || [ "$STEAM_DEBUGGER" == "cgdb" ]; then
 		unset LD_PRELOAD
 	fi
 
+	echo $STEAM_DEBUGGER -x "$ARGSFILE" --args "$STEAMROOT/$STEAMEXEPATH" "$@"
 	$STEAM_DEBUGGER -x "$ARGSFILE" --args "$STEAMROOT/$STEAMEXEPATH" "$@"
 	rm "$ARGSFILE"
 elif [ "$STEAM_DEBUGGER" == "valgrind" ]; then
-    : "${STEAM_VALGRIND:=}"
+	: "${STEAM_VALGRIND:=}"
 	DONT_BREAK_ON_ASSERT=1 G_SLICE=always-malloc G_DEBUG=gc-friendly valgrind --error-limit=no --undef-value-errors=no --suppressions=$PLATFORM/steam.supp $STEAM_VALGRIND "$STEAMROOT/$STEAMEXEPATH" "$@" 2>&1 | tee steam_valgrind.txt
 elif [ "$STEAM_DEBUGGER" == "callgrind" ]; then
-    valgrind --tool=callgrind --instr-atstart=no "$STEAMROOT/$STEAMEXEPATH" "$@"
+	valgrind --tool=callgrind --instr-atstart=no "$STEAMROOT/$STEAMEXEPATH" "$@"
 elif [ "$STEAM_DEBUGGER" == "strace" ]; then
-    strace -osteam.strace "$STEAMROOT/$STEAMEXEPATH" "$@"
+	strace -osteam.strace "$STEAMROOT/$STEAMEXEPATH" "$@"
 else
+	echo $STEAM_DEBUGGER "$STEAMROOT/$STEAMEXEPATH" "$@"
 	$STEAM_DEBUGGER "$STEAMROOT/$STEAMEXEPATH" "$@"
 fi
 STATUS=$?
